@@ -97,10 +97,14 @@ dv_decoder_new(int add_ntsc_setup, int clamp_luma, int clamp_chroma) {
 	
   result->video = dv_video_new();
   if(!result->video) goto no_video;
+  result->video->dv_decoder = result;
 
   result->audio = dv_audio_new();
   if(!result->audio) goto no_audio;
+  result->audio->dv_decoder = result;
 
+  dv_set_error_log (result, stderr);
+  dv_set_audio_correction (result, DV_AUDIO_CORRECT_AVERAGE);
 #if HAVE_LIBPOPT
   result->option_table[DV_DECODER_OPT_SYSTEM] = (struct poptOption) {
     longName: "video-system", 
@@ -390,9 +394,9 @@ static inline void
 dv_render_macroblock_yuv(dv_decoder_t *dv, dv_macroblock_t *mb, uint8_t **pixels, int *pitches) {
   if(dv->sampling == e_dv_sample_411) {
     if(mb->x >= 704) {
-      dv_mb411_right_YUY2(mb, pixels, pitches); /* Right edge are 420! */
+      dv_mb411_right_YUY2(mb, pixels, pitches, dv->add_ntsc_setup); /* Right edge are 420! */
     } else {
-      dv_mb411_YUY2(mb, pixels, pitches);
+      dv_mb411_YUY2(mb, pixels, pitches, dv->add_ntsc_setup);
     } /* else */
   } else {
     DV_MB420_YUV(mb, pixels, pitches);
@@ -436,7 +440,7 @@ void dv_check_coeff_ranges(dv_macroblock_t *mb) {
 }
 
 void
-dv_decode_full_frame(dv_decoder_t *dv, uint8_t *buffer, 
+dv_decode_full_frame(dv_decoder_t *dv, const uint8_t *buffer, 
 		     dv_color_space_t color_space, uint8_t **pixels, int *pitches) {
 
   static dv_videosegment_t vs;
@@ -474,7 +478,7 @@ dv_decode_full_frame(dv_decoder_t *dv, uint8_t *buffer,
       } /* if */
       /* stage 1: parse and VLC decode 5 macroblocks that make up a video segment */
       offset = dif * 80;
-      bitstream_new_buffer(seg->bs, buffer + offset, 80*5); 
+      bitstream_new_buffer(seg->bs, (uint8_t *)buffer + offset, 80*5); 
       dv_parse_video_segment(seg, dv->quality);
       /* stage 2: dequant/unweight/iDCT blocks, and place the macroblocks */
       dif+=5;
@@ -530,8 +534,18 @@ dv_decode_full_frame(dv_decoder_t *dv, uint8_t *buffer,
   exit(-1);
 } /* dv_decode_full_frame  */
 
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_frame_has_audio_errors (dv_decoder_t *dv)
+{
+  return (dv -> audio -> block_failure);
+} /* dv_has_audio_errors */
+
+/* ---------------------------------------------------------------------------
+ */
 int 
-dv_decode_full_audio(dv_decoder_t *dv, uint8_t *buffer, int16_t **outbufs)
+dv_decode_full_audio(dv_decoder_t *dv, const uint8_t *buffer, int16_t **outbufs)
 {
   int ds, dif, audio_dif, result;
   int ch;
@@ -539,7 +553,7 @@ dv_decode_full_audio(dv_decoder_t *dv, uint8_t *buffer, int16_t **outbufs)
   dif=0;
 
   if(!dv_parse_audio_header(dv, buffer)) goto no_audio;
-
+  dv->audio->block_failure = dv->audio->sample_failure = 0;
   for (ds=0; ds < dv->num_dif_seqs; ds++) {
     dif += 6;
     for(audio_dif=0; audio_dif<9; audio_dif++) {
@@ -548,6 +562,24 @@ dv_decode_full_audio(dv_decoder_t *dv, uint8_t *buffer, int16_t **outbufs)
     } /* for  */
   } /* for */
 
+  if (dv -> audio -> block_failure) {
+    if (dv -> audio -> error_log) {
+      fprintf (dv -> audio -> error_log,
+               "# audio block failure for %d blocks = %d samples of %d\n",
+               dv -> audio -> block_failure,
+               dv -> audio -> sample_failure,
+               dv -> audio -> samples_this_frame);
+    }
+    dv_audio_correct_errors (dv -> audio, outbufs);
+  }
+  if (!dv->audio->block_failure && dv->audio->sample_failure) {
+    if (dv -> audio -> error_log) {
+      fprintf (dv -> audio -> error_log,
+               "# sample failure without block failure: "
+                 "report this to libdv at SF!!\n");
+    }
+  }
+
   if(dv->audio->emphasis) {
     for(ch=0; ch< dv->audio->num_channels; ch++) {
       dv_audio_deemphasis(dv->audio, outbufs[ch]);
@@ -562,11 +594,51 @@ dv_decode_full_audio(dv_decoder_t *dv, uint8_t *buffer, int16_t **outbufs)
   
 } /* dv_decode_full_audio */
 
+/* ---------------------------------------------------------------------------
+ */
+void
+dv_report_video_error (dv_decoder_t *dv, uint8_t *data)
+{
+    int  i,
+         error_code;
+    char err_msg1 [40],
+         err_msg2 [40];
 
-/*
- * query functions based upon vaux data
+  if (!dv -> video -> error_log)
+    return;
+  /* -------------------------------------------------------------------------
+   * got through all packets
+   */
+  for (i = 0; i < dv -> frame_size; i += 80) {
+    /* -----------------------------------------------------------------------
+     * check that's a video packet
+     */
+    if ((data [i] & 0xe0) == 0x80) {
+      error_code = data [i + 3] >> 4;
+      if (error_code) {
+        dv_get_timestamp (dv, err_msg1);
+        dv_get_recording_datetime (dv, err_msg2);
+        fprintf (dv -> video -> error_log,
+                 "%s %s %s %02x %02x %02x %02x\n",
+/*                 (error_code < 8) ? "vel" : "veh", */
+                 "ver",
+                 err_msg1,
+                 err_msg2,
+                 data [i + 0],
+                 data [i + 1],
+                 data [i + 2],
+                 data [i + 3]);
+      }
+    }
+  }
+}
+
+/* ---------------------------------------------------------------------------
+ * query functions based upon VAUX data
  */
 
+/* ---------------------------------------------------------------------------
+ */
 int
 dv_get_vaux_pack (dv_decoder_t *dv, uint8_t pack_id, uint8_t *data)
 {
@@ -577,6 +649,8 @@ dv_get_vaux_pack (dv_decoder_t *dv, uint8_t pack_id, uint8_t *data)
   return 0;
 } /* dv_get_vaux_pack */
 
+/* ---------------------------------------------------------------------------
+ */
 int
 dv_frame_is_color (dv_decoder_t *dv)
 {
@@ -591,6 +665,8 @@ dv_frame_is_color (dv_decoder_t *dv)
   return -1;
 }
 
+/* ---------------------------------------------------------------------------
+ */
 int
 dv_system_50_fields (dv_decoder_t *dv)
 {
@@ -605,6 +681,8 @@ dv_system_50_fields (dv_decoder_t *dv)
   return -1;
 }
 
+/* ---------------------------------------------------------------------------
+ */
 int
 dv_format_normal (dv_decoder_t *dv)
 {
@@ -619,6 +697,8 @@ dv_format_normal (dv_decoder_t *dv)
   return -1;
 }
 
+/* ---------------------------------------------------------------------------
+ */
 int
 dv_format_wide (dv_decoder_t *dv)
 {
@@ -633,6 +713,8 @@ dv_format_wide (dv_decoder_t *dv)
   return -1;
 }
 
+/* ---------------------------------------------------------------------------
+ */
 int
 dv_frame_changed (dv_decoder_t *dv)
 {
@@ -647,4 +729,139 @@ dv_frame_changed (dv_decoder_t *dv)
   return -1;
 }
 
+/* ---------------------------------------------------------------------------
+ * query functions based upon SSYB data
+ * decoding code is taken from kino
+ */
+
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_get_timestamp (dv_decoder_t *dv, char *tstptr)
+{
+    int  id;
+
+  if ((id = dv -> ssyb_pack [0x13]) != 0xff) {
+    sprintf (tstptr,
+             "%02d:%02d:%02d.%02d",
+             ((dv -> ssyb_data [id] [3] >> 4) & 0x03) * 10 +
+              (dv -> ssyb_data [id] [3] & 0x0f),
+             ((dv -> ssyb_data [id] [2] >> 4) & 0x07) * 10 +
+              (dv -> ssyb_data [id] [2] & 0x0f),
+             ((dv -> ssyb_data [id] [1] >> 4) & 0x07) * 10 +
+              (dv -> ssyb_data [id] [1] & 0x0f),
+             ((dv -> ssyb_data [id] [0] >> 4) & 0x03) * 10 +
+              (dv -> ssyb_data [id] [0] & 0x0f));
+
+    return 1;
+  }
+  strcpy (tstptr, "00:00:00.00");
+  return 0;
+}
+
+/* ---------------------------------------------------------------------------
+ * timecode is an array of 4 ints
+ */
+int
+dv_get_timestamp_int (dv_decoder_t *dv, int *timestamp)
+{
+    int  id;
+
+  if ((id = dv -> ssyb_pack [0x13]) != 0xff) {
+    timestamp[0] = ((dv -> ssyb_data [id] [3] >> 4) & 0x03) * 10 +
+              (dv -> ssyb_data [id] [3] & 0x0f);
+	  
+    timestamp[1] = ((dv -> ssyb_data [id] [2] >> 4) & 0x07) * 10 +
+              (dv -> ssyb_data [id] [2] & 0x0f);
+	  
+    timestamp[2] = ((dv -> ssyb_data [id] [1] >> 4) & 0x07) * 10 +
+              (dv -> ssyb_data [id] [1] & 0x0f);
+	  
+    timestamp[3] = ((dv -> ssyb_data [id] [0] >> 4) & 0x03) * 10 +
+              (dv -> ssyb_data [id] [0] & 0x0f);
+
+    return 1;
+  }
+  return 0;
+}
+
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_get_recording_datetime (dv_decoder_t *dv, char *dtptr)
+{
+    int  id1, id2, year;
+
+  if ((id1 = dv -> ssyb_pack [0x62]) != 0xff &&
+      (id2 = dv -> ssyb_pack [0x63]) != 0xff) {
+    year = dv -> ssyb_data [id1] [3];
+    year = (year & 0x0f) + 10 * ((year >> 4) & 0x0f);
+    year += (year < 25) ? 2000 : 1900;
+    sprintf (dtptr,
+             "%04d-%02d-%02d %02d:%02d:%02d",
+             year,
+             ((dv -> ssyb_data [id1] [2] >> 4) & 0x01) * 10 +
+               (dv -> ssyb_data [id1] [2] & 0x0f),
+             ((dv -> ssyb_data [id1] [1] >> 4) & 0x03) * 10 +
+               (dv -> ssyb_data [id1] [1] & 0x0f),
+             ((dv -> ssyb_data [id2] [3] >> 4) & 0x03) * 10 +
+              (dv -> ssyb_data [id2] [3] & 0x0f),
+             ((dv -> ssyb_data [id2] [2] >> 4) & 0x07) * 10 +
+              (dv -> ssyb_data [id2] [2] & 0x0f),
+             ((dv -> ssyb_data [id2] [1] >> 4) & 0x07) * 10 +
+              (dv -> ssyb_data [id2] [1] & 0x0f));
+    return 1;
+  }
+  strcpy (dtptr, "0000-00-00 00:00:00");
+  return 0;
+}
+
+/* ---------------------------------------------------------------------------
+ */
+int
+dv_get_recording_datetime_tm (dv_decoder_t *dv, struct tm *rec_dt)
+{
+    int  id1, id2, year;
+
+  if ((id1 = dv -> ssyb_pack [0x62]) != 0xff &&
+      (id2 = dv -> ssyb_pack [0x63]) != 0xff) {
+    year = dv -> ssyb_data [id1] [3];
+    year = (year & 0x0f) + 10 * ((year >> 4) & 0x0f);
+    year += (year < 25) ? 2000 : 1900;
+    rec_dt -> tm_isdst = rec_dt -> tm_yday = rec_dt -> tm_wday = -1;
+    rec_dt -> tm_year = year - 1900;
+    rec_dt -> tm_mon = ((dv -> ssyb_data [id1] [2] >> 4) & 0x01) * 10 +
+                        (dv -> ssyb_data [id1] [2] & 0x0f) - 1;
+    rec_dt -> tm_mday = ((dv -> ssyb_data [id1] [1] >> 4) & 0x03) * 10 +
+                         (dv -> ssyb_data [id1] [1] & 0x0f);
+    rec_dt -> tm_hour = ((dv -> ssyb_data [id2] [3] >> 4) & 0x03) * 10 +
+                         (dv -> ssyb_data [id2] [3] & 0x0f);
+    rec_dt -> tm_min  = ((dv -> ssyb_data [id2] [2] >> 4) & 0x07) * 10 +
+                         (dv -> ssyb_data [id2] [2] & 0x0f);
+    rec_dt -> tm_sec  = ((dv -> ssyb_data [id2] [1] >> 4) & 0x07) * 10 +
+                         (dv -> ssyb_data [id2] [1] & 0x0f);
+    /* -----------------------------------------------------------------------
+     * sanity check
+     */
+    if (mktime (rec_dt) == -1)
+      return 0;
+
+    return 1;
+  }
+  return 0;
+}
+
+/* ---------------------------------------------------------------------------
+ */
+FILE *
+dv_set_error_log (dv_decoder_t *dv, FILE *log)
+{
+    FILE *old_log;
+    
+  old_log = dv -> audio -> error_log;
+  dv -> audio -> error_log =
+    dv -> video -> error_log =
+      log;
+  return old_log;
+}
 /*@}*/
