@@ -1,4 +1,4 @@
-/* 
+/*
  *  encode.c
  *
  *     Copyright (C) James Bowman  - July 2000
@@ -1490,6 +1490,7 @@ dv_encoder_new(int rem_ntsc_setup, int clamp_luma, int clamp_chroma) {
 
   dv_init( clamp_luma, clamp_chroma);
 
+  result->frame_count = 0;
   return(result);
   
 no_cb:
@@ -1575,7 +1576,7 @@ int dv_encode_videosegment( dv_encoder_t *dv_enc,
 			"specified: %d!\n", dv_enc->vlc_encode_passes);
 		exit(-1);
 	}
-		
+
 	for (m = 0, mb = videoseg->mb; m < 5; m++, mb++) {
 		put_bits(vsbuffer, (8 * (80 * m)) + 28, 4, mb->qno);
 		
@@ -1746,7 +1747,7 @@ int dv_encode_full_frame(dv_encoder_t *dv_enc, uint8_t **in,
 		   the VLC data */
 		/* Loop through video segments */
 		for (v = 0; v < 27; v++) {
-			/* skip audio block - 
+			/* skip audio block -
 			   interleaved before every 3rd video segment
 			*/
 
@@ -1767,13 +1768,14 @@ int dv_encode_full_frame(dv_encoder_t *dv_enc, uint8_t **in,
 		} 
 	} 
 	
-	write_meta_data(target, 0, dv_enc->isPAL, dv_enc->is16x9, &now);
+	write_meta_data(target, dv_enc->frame_count++, dv_enc->isPAL, dv_enc->is16x9, &now);
 
 	return 0;
 }
 
-void swab(void*, void*, ssize_t);
-
+#ifdef __linux__
+void swab(const void*, void*, ssize_t);
+#endif
 
 /** @brief Encode signed 16-bit integer PCM audio data into a frame of DV video.
  *
@@ -1811,3 +1813,4 @@ int dv_encode_full_audio(dv_encoder_t *dv_enc, int16_t **pcm,
 
 	return raw_insert_audio(frame_buf, &audio, dv_enc->isPAL);
 }
+
