@@ -1,4 +1,4 @@
-/* 
+/*
  *  display.c
  *
  *     Copyright (C) Charles 'Buck' Krasic - April 2000
@@ -25,7 +25,7 @@
  */
 
 /* Most of this file is derived from patches 101018 and 101136 submitted by
- * Stefan Lucke <lucke@berlin.snafu.de> */
+ * Stefan Lucke <Stefan.Lucke1@epost.de> */
 
 
 #include <unistd.h>
@@ -47,6 +47,8 @@
 #include <popt.h>
 #endif
 
+static dv_display_t *_dv_dpy;
+
 static int      dv_display_SDL_init(dv_display_t *dv_dpy, gchar *w_name, gchar   *i_name           );
 static gboolean dv_display_gdk_init(dv_display_t *dv_dpy, gint  *argc,   gchar ***argv             );
 
@@ -231,9 +233,6 @@ dv_display_exit(dv_display_t *dv_dpy) {
     if(dv_dpy->shminfo.shmaddr)
       shmdt(dv_dpy->shminfo.shmaddr);
 
-    if(dv_dpy->shminfo.shmid > 0)
-      shmctl(dv_dpy->shminfo.shmid, IPC_RMID, 0);
-
     if(dv_dpy->xv_image)
       free(dv_dpy->xv_image);
 #endif /* HAVE_LIBXV */
@@ -577,6 +576,9 @@ dv_display_Xv_init(dv_display_t *dv_dpy, gchar *w_name, gchar *i_name,
     shmat(dv_dpy->shminfo.shmid, 0, 0);
 
   XShmAttach(dv_dpy->dpy, &dv_dpy->shminfo);
+  if (dv_dpy -> shminfo. shmid > 0)
+    shmctl (dv_dpy -> shminfo. shmid, IPC_RMID, 0);
+
   XSync(dv_dpy->dpy, False);
 
   return 1;
@@ -667,12 +669,12 @@ dv_display_SDL_init(dv_display_t *dv_dpy, gchar *w_name, gchar *i_name) {
 
 #endif /* HAVE_SDL */
 
+/* ---------------------------------------------------------------------------
+ */
 static void
-dv_display_exit_handler(int code, void *arg)
+dv_display_exit_handler (void)
 {
-  if(code && arg) {
-    dv_display_exit(arg);
-  } /* if */
+  dv_display_exit (_dv_dpy);
 } /* dv_display_exit_handler */
 
 
@@ -804,7 +806,8 @@ dv_display_init(dv_display_t *dv_dpy, gint *argc, gchar ***argv, gint width, gin
   fprintf(stderr, " Using gtk for display\n");
 
  ok:
-  on_exit(dv_display_exit_handler, dv_dpy);
+  _dv_dpy = dv_dpy;
+  atexit(dv_display_exit_handler);
   return(TRUE);
 
  fail:
